/*
* Copyright (c) 2005-08, Craig Bishop and Colin Graeme Johnson
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the names of Craig Bishop and/or Colin Graeme Johnson, 
*       nor the name of the University of Kent may be used to endorse 
*       or promote products derived from this software without
*       specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY COLIN GRAEME JOHNSON ``AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL COLIN GRAEME JOHNSON BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/*
 * Created on 10-Jun-2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package main;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Set;

import main.progAnal.MethodGetter;
import main.progAnal.StatementAnalyser;
import main.progAnal.RoleHolder;
import main.progAnal.SourceSorter;
import main.progAnal.StatementGetter;
import main.progAnal.ProgramSlicer;
import main.rules.RuleApplyer;

/**
 * @author cbishop
 * 
 * Main class to run role analyser program
 */
public class RoleAnalyser {

	private SourceSorter sourceSorter;

	private LinkedHashMap brokenSource;

	private StatementGetter statementGetter;

	private static final String[] PROGRAMS = { "BubbleSort.java",
			"Closest.java", "DiceGame.java", "DivMod7.java", "Doubles.java",
			"Fibonachi.java", "Growth.java", "Histogram.java", "Lexical.java",
			"Multiplication.java", "Number.java", "Occur.java",
			"ProgramTime.java", "Saw.java", "SmoothedAverage.java",
			"Square.java", "TwoLargest.java" };

	private static final String[] ROLES = { "Fixed Value", "Organizer",
			"Stepper", "Most Recent Holder", "Gatherer", "Most Wanted Holder",
			"One Way Flag", "Transformation", "Follower", "Temporary" };

	private HashMap roleMap;

	private File[] files;

	public RoleAnalyser(File[] inputFiles) //should input name of file to check
	// here
	{
		files = inputFiles;
	}

	/**
	 * Check roles played by variables
	 * 
	 * @return HashMap containing results of role checking
	 */
	public HashMap checkRoles() {
		HashMap returnMap = new HashMap();
		for (int i = 0; i < files.length; i++) {
			ArrayList fileResults = new ArrayList();
			String fileName = files[i].toString();
			roleMap = initialiseMap();
			brokenSource = getSortedSource(fileName);
			//printer.printSource(brokenSource);//debug
			Set variables = getVariables();
			LinkedHashMap variableStatements = getStatementMap(brokenSource,
					variables);
			RoleHolder roleHolder = sourceSorter.getRoleHolder();
			LinkedHashMap roles = roleHolder.getRoles();
			HashMap sortedStatements = sortStatements(variables,
					variableStatements);
			HashMap analysedStatements = getStatementAnalysis(sortedStatements,
					variables);
			//printer.printAnalysedStatements(analysedStatements); //debug
			ArrayList methods = getMethods(sortedStatements);
			HashMap checkedRoles = getResults(analysedStatements, roles,
					methods);
			fileResults.add(checkedRoles); //add results of analysis for given
			// file
			fileResults.add(roles); //add annotated roles for each variable in
			// given file
			returnMap.put(files[i].getName(), fileResults); //add fileResults
			// to return map for
			// given file
		}
		returnMap.put("role map", roleMap);//add role map to return map
		return returnMap;
	}

	/*
	 * Return hierarchy of source code statements
	 */
	private LinkedHashMap getSortedSource(String fileName) {
		sourceSorter = new SourceSorter();
		LinkedHashMap returnMap = sourceSorter.sortSource(fileName);
		return returnMap;
	}

	/*
	 * Return Set of variables
	 */
	private Set getVariables() {
		RoleHolder roles = sourceSorter.getRoleHolder();
		Set returnSet = roles.getVariables();
		return returnSet;
	}

	/*
	 * Return map of variable statements
	 */
	private LinkedHashMap getStatementMap(LinkedHashMap sourceMap, Set variables) {
		StatementGetter statementGetter = new StatementGetter();
		LinkedHashMap returnMap = statementGetter.getStatements(brokenSource,
				variables);
		return returnMap;
	}

	/*
	 * Return map of program slices
	 */
	private HashMap sortStatements(Set variables,
			LinkedHashMap relevantStatements) {
		ProgramSlicer statementSorter = new ProgramSlicer();
		HashMap returnMap = statementSorter.sortStatements(variables,
				relevantStatements);
		return returnMap;
	}

	/*
	 * Return map of analysed statements
	 */
	private HashMap getStatementAnalysis(HashMap statements, Set variables) {
		HashMap analysedStatements = new HashMap();
		StatementAnalyser statementAnalyser = new StatementAnalyser(statements,
				variables);
		analysedStatements = statementAnalyser.getStatementAnalysis();
		return analysedStatements;
	}

	/*
	 * Return map of results from role checking
	 */
	private HashMap getResults(HashMap analysedStatements, LinkedHashMap roles,
			ArrayList methods) {
		HashMap checkedRoles = new HashMap();
		RuleApplyer ruleApplyer = new RuleApplyer();
		checkedRoles = ruleApplyer.applyRules(analysedStatements, roles,
				methods);
		return checkedRoles;
	}

	/*
	 * Initialised map with roles and associated int values
	 */
	private HashMap initialiseMap() {
		HashMap returnMap = new HashMap();
		for (int i = 0; i < 10; i++) {
			returnMap.put(new Integer(i + 1), ROLES[i]);
		}
		return returnMap;
	}

	/*
	 * Return list of methods in analysed program
	 */
	private ArrayList getMethods(HashMap sortedStatements) {
		ArrayList returnArray = new ArrayList();
		MethodGetter methodGetter = new MethodGetter();
		returnArray = methodGetter.getMethods(sortedStatements);
		return returnArray;
	}
}