/*
* Copyright (c) 2005-08, Craig Bishop and Colin Graeme Johnson
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the names of Craig Bishop and/or Colin Graeme Johnson, 
*       nor the name of the University of Kent may be used to endorse 
*       or promote products derived from this software without
*       specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY COLIN GRAEME JOHNSON ``AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL COLIN GRAEME JOHNSON BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/*
 * Created on 29-Jun-2005
 */
package main.progAnal;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Set;

/**
 * Class to produce program slice for each variable
 * 
 * @author cbishop
 */
public class ProgramSlicer {

	HashMap statementMap;

	Iterator iter;

	/**
	 * Return map containing program slices for each variable
	 * 
	 * @param variables
	 *            Set of variable name
	 * @param relevantStatements
	 *            LinkedHashMap containing statement for program slice
	 * @return HashMap of program slices
	 */
	public HashMap sortStatements(Set variables,
			LinkedHashMap relevantStatements) {
		statementMap = new HashMap();
		iter = variables.iterator();
		while (iter.hasNext()) {
			String keyString = (String) iter.next();
			ArrayList parentArray = (ArrayList) relevantStatements
					.get(keyString);
			ArrayList variableArray = getVariableStatements(parentArray);
			statementMap.put(keyString, variableArray);
		}
		return statementMap;
	}

	/*
	 * Return ArrayList of program slice statements for each variable
	 */
	private ArrayList getVariableStatements(ArrayList inputArray) {
		ArrayList sortedArray = new ArrayList();
		ArrayList previousControlStatements = new ArrayList();
		int bracketCount = 0;
		for (int i = 0; i < inputArray.size(); i++) {
			ArrayList statementArray = (ArrayList) inputArray.get(i);
			int startIndex = 0;
			for (int count = 0; count < statementArray.size(); count++) {
				String statementString = (String) (statementArray.get(count));
				if (!previous(statementString, previousControlStatements,
						startIndex)) {
					if (statementString.endsWith("}")) {
						bracketCount--;
						previousControlStatements.add(statementString);
						startIndex++;
					}
					sortedArray.add(statementString);
					if (statementString.endsWith("{")) {
						bracketCount++;
						for (int j = startIndex; j < previousControlStatements
								.size(); j++) {
							previousControlStatements.remove(startIndex);
							sortedArray.add("}");
							bracketCount--;
						}
						previousControlStatements.add(statementString);
						startIndex++;
					}
				} else {
					startIndex++; //so don't remove relevant statement from
								  // previous control statements
				}
			}
			if (i == inputArray.size() - 1) {
				for (int k = 0; k < bracketCount; k++) {
					sortedArray.add("}");
				}
			}
		}
		return sortedArray;
	}

	/*
	 * Return whether control statement has already been added to program slice
	 */
	private boolean previous(String statement, ArrayList previousStatements,
			int start) {
		boolean isPrevious = false;
		if (previousStatements.size() > 0) {
			for (int prevIndex = 0; prevIndex < previousStatements.size(); prevIndex++) {
				String tempString = (String) previousStatements.get(prevIndex);
				if (statement.equals(tempString) && start <= prevIndex) {
					isPrevious = true;
					break;
				}
			}
		}
		return isPrevious;
	}
}