/*
* Copyright (c) 2005-08, Craig Bishop and Colin Graeme Johnson
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the names of Craig Bishop and/or Colin Graeme Johnson, 
*       nor the name of the University of Kent may be used to endorse 
*       or promote products derived from this software without
*       specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY COLIN GRAEME JOHNSON ``AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL COLIN GRAEME JOHNSON BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package main.progAnal;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.LinkedHashMap;
import java.util.Stack;

/**
 * Class to take java source code file as input and output ArrayList containing
 * hierarchical breakdown of code into methods, branches, sub-branches, etc
 * 
 * @author Craig Bishop
 * @version 090607
 */
public class SourceSorter {
	private LinkedHashMap currentMap;

	private Stack hashMaps;

	private String inputLine;

	private int bracketCount;

	private boolean openQuote;

	private RoleHolder roles;

	/**
	 * Constructor for objects of class SourceBreakDown
	 */
	public SourceSorter() {
	}

	/**
	 * Method to break down input file into ArrayList of Strings with separate
	 * entry in list for each level in the code hierarchy.
	 * 
	 * @param fileName
	 *            String giving name of file to be checked
	 * @return LinkedHashMap containing a breakdown of the input source code
	 */
	public LinkedHashMap sortSource(String fileName) {
		roles = new RoleHolder();
		currentMap = new LinkedHashMap();
		hashMaps = new Stack();
		inputLine = "";
		String tempLine;
		boolean openQuote = false;
		try {
			FileInputStream fileIn = new FileInputStream(fileName);
			BufferedReader input = new BufferedReader(new InputStreamReader(
					fileIn));
			do {
				tempLine = input.readLine();
				if (tempLine != null) {
					String trimLine = tempLine.trim();
					processLine(trimLine);
				}
			} while (tempLine != null);
		} catch (FileNotFoundException e) {
			System.out.println(e.getMessage());
		} catch (IOException e) {
			System.out.println(e.getMessage());
		}
		return currentMap;
	}

	/**
	 * Method to process each line for addition to LinkedHashMap
	 * 
	 * @param lineToProcess
	 *            the line to be processed.
	 */
	private void processLine(String lineToProcess) {
		for (int i = 0; i < lineToProcess.length(); i++) {
			inputLine += lineToProcess.substring(i, i + 1); //add first
															// character of
															// string to
															// inputLine
			if (lineToProcess.charAt(i) == 34) {
				openQuote = !openQuote;
			}
			if (!openQuote) { //check that string to be processed is not within
							  // inverted commas.
				if (lineToProcess.substring(i).startsWith("*/")) { //check if
																   // previous
																   // string is
																   // comment
					inputLine = ""; //remove comment string prior to "*/"
					i++; // increment i so that "/" is not added to input line
						 // next time round
				} else if (comment(lineToProcess.substring(i))) { //check if
																  // subsequent
																  // string is
																  // comment
					String commentLine = lineToProcess.substring(i);
					if (roleDeclaration(commentLine)) {
						roles.noteRole(commentLine);
					}
					if (commentLine.startsWith("//")) {
						inputLine = inputLine.substring(0,
								inputLine.length() - 1); // remove first
														 // character from
														 // inputLine
						i = lineToProcess.length();
					}
				} else if (lineToProcess.substring(i, i + 1).equals("(")) { //check
																			// if
																			// character
																			// within
																			// brackets
					bracketCount++;
				} else if (lineToProcess.substring(i, i + 1).equals(")")) {
					bracketCount--;
				} else if (lineToProcess.substring(i, i + 1).equals(";")
						&& bracketCount == 0) { //only treat ";" as end line if
												// not within brackets
					inputLine = inputLine.trim();
					currentMap.put(inputLine, inputLine);
					inputLine = "";
				} else if (lineToProcess.substring(i, i + 1).equals("{")
						&& !lineToProcess.endsWith("};")) {
					LinkedHashMap tempMap = new LinkedHashMap();
					inputLine = inputLine.trim();
					currentMap.put(inputLine, tempMap);
					hashMaps.push(currentMap);
					currentMap = (LinkedHashMap) currentMap.get(inputLine);
					inputLine = "";
				} else if (lineToProcess.substring(i, i + 1).equals("}")
						&& !lineToProcess.endsWith("};")) {
					inputLine = inputLine.trim();
					currentMap.put(inputLine, inputLine);
					currentMap = (LinkedHashMap) hashMaps.pop();
					inputLine = "";
				}
			}
		}
	}

	/*
	 * Return whether line is comment
	 */
	private boolean comment(String lineString) {
		boolean isComment = false;
		if (lineString.startsWith("/*") || lineString.startsWith("//")) {
			isComment = true;
		}
		return isComment;
	}

	/*
	 * Return whether role is declared for variable in comment line
	 */
	private boolean roleDeclaration(String commentLine) {
		boolean roleDeclared = false;
		for (int i = 0; i < commentLine.length() - 1; i++) {
			if (commentLine.substring(i, i + 2).equals("%%")) {
				roleDeclared = true;
			}
		}
		return roleDeclared;
	}

	public RoleHolder getRoleHolder() {
		return roles;
	}
}

