/*
* Copyright (c) 2005-08, Craig Bishop and Colin Graeme Johnson
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the names of Craig Bishop and/or Colin Graeme Johnson, 
*       nor the name of the University of Kent may be used to endorse 
*       or promote products derived from this software without
*       specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY COLIN GRAEME JOHNSON ``AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL COLIN GRAEME JOHNSON BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/*
 * Created on 05-Jul-2005
 *
 */
package main.rules;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Set;

/**
 * @author cbishop
 */
public class RuleApplyer {

	/**
	 * Return map of results from role checking
	 * 
	 * @param analysedStatements
	 *            HashMap containing whereabouts of variable statements
	 * @param roles
	 *            Map containing variables and their declared roles
	 * @return HashMap containing checked roles for each variable
	 */
	public HashMap applyRules(HashMap analysedStatements, LinkedHashMap roles,
			ArrayList methods) {
		HashMap checkedRoles = new HashMap();
		ArrayList checkedRole = new ArrayList();
		Set variableSet = roles.keySet();
		Iterator it = variableSet.iterator();
		while (it.hasNext()) {
			String variable = (String) it.next();
			Integer roleValue = (Integer) roles.get(variable);
			LinkedHashMap analysedMap = (LinkedHashMap) analysedStatements
					.get(variable);
			RoleChecker roleChecker;
			switch (roleValue.intValue()) {
			case 1:
				roleChecker = new FixedValue(analysedMap, variable, methods);
				break;
			case 2:
				roleChecker = new Organizer(analysedMap, variable, methods);
				break;
			case 3:
				roleChecker = new Stepper(analysedMap, variable, methods);
				break;
			case 4:
				roleChecker = new MostRecentHolder(analysedMap, variable,
						methods);
				break;
			case 5:
				roleChecker = new Gatherer(analysedMap, variable, methods);
				break;
			case 6:
				roleChecker = new MostWantedHolder(analysedMap, variable,
						methods);
				break;
			case 7:
				roleChecker = new OneWayFlag(analysedMap, variable, methods);
				break;
			case 8:
				roleChecker = new Transformation(analysedMap, variable, methods);
				break;
			case 9:
				roleChecker = new Follower(analysedMap, variable, methods);
				break;
			case 10:
				roleChecker = new Temporary(analysedMap, variable, methods);
				break;
			default:
				roleChecker = new OtherRole(analysedMap, variable, methods);
			}
			checkedRole = roleChecker.checkRole();
			checkedRoles.put(variable, checkedRole);
		}
		return checkedRoles;
	}
}